/**
 * eobjects.org MetaModel
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
package org.eobjects.metamodel.couchdb;

import java.util.Iterator;
import java.util.Map;

import org.ektorp.CouchDbConnector;
import org.ektorp.Page;
import org.ektorp.ViewQuery;
import org.eobjects.metamodel.data.AbstractDataSet;
import org.eobjects.metamodel.data.DefaultRow;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.SelectItem;

/**
 * DataSet implementation for couch db.
 */
final class CouchDbDataSet extends AbstractDataSet {

	private final SelectItem[] _selectItems;
	private final CouchDbConnector _connector;
	private final ViewQuery _query;

	@SuppressWarnings("rawtypes")
	private Page<Map> _page;

	@SuppressWarnings("rawtypes")
	private Iterator<Map> _iterator;
	private DefaultRow _row;

	public CouchDbDataSet(SelectItem[] selectItems, CouchDbConnector connector, ViewQuery query,
			@SuppressWarnings("rawtypes") Page<Map> page) {
		_selectItems = selectItems;
		_connector = connector;
		_query = query;
		_page = page;
		_iterator = _page.iterator();
	}

	@Override
	public SelectItem[] getSelectItems() {
		return _selectItems;
	}

	@Override
	public boolean next() {
		if (_iterator == null || !_iterator.hasNext()) {
			if (_page.isHasNext()) {
				_page = _connector.queryForPage(_query, _page.getNextPageRequest(), Map.class);
				_iterator = _page.iterator();
				return next();
			}
			_row = null;
			return false;
		}

		Map<?, ?> map = _iterator.next();
		final Object[] values = new Object[_selectItems.length];
		for (int i = 0; i < _selectItems.length; i++) {
			final String key = _selectItems[i].getColumn().getName();
			final Object value = map.get(key);
			values[i] = value;
		}

		_row = new DefaultRow(_selectItems, values);

		return true;
	}

	@Override
	public Row getRow() {
		return _row;
	}

}
